var RE = {};

RE.width;
RE.height;

RE.editor = document.getElementById('editor');

RE.selection = {};

RE.selection.selectionStartRange = null;
RE.selection.selectionEndRange = null;

RE.selection.lastTouchPoint = null;

RE.callback = function() {
    window.location.href = "re-callback://" + encodeURI(RE.getHtml());
}

RE.editor.addEventListener("input", RE.callback);

RE.setHtml = function(contents) {
    RE.editor.innerHTML = contents;
}

RE.getHtml = function() {
    return RE.editor.innerHTML;
}

RE.getText = function() {
    return RE.editor.innerText;
}

RE.setFontSize = function(size) {
    RE.editor.style.fontSize = size;
}

RE.setBackgroundColor = function(color) {
    RE.editor.style.backgroundColor = color;
}

RE.setWidth = function(size) {
    RE.editor.style.minWidth = size;
    RE.width = size;
}

RE.setHeight = function(size) {
    RE.editor.style.minHeight = size;
    RE.height = size;
}

RE.setPlaceholder = function(placeholder) {
    RE.editor.setAttribute("placeholder", placeholder);
}

RE.undo = function() {
	document.execCommand('undo', false, null);
}

RE.redo = function() {
	document.execCommand('redo', false, null);
}

RE.setBold = function() {
    document.execCommand('bold', false, null);
}

RE.setItalic = function() {
    document.execCommand('italic', false, null);
}

RE.setSubscript = function() {
    document.execCommand('subscript', false, null);
}

RE.setSuperscript = function() {
    document.execCommand('superscript', false, null);
}

RE.setStrikeThrough = function() {
    document.execCommand('strikeThrough', false, null);
}

RE.setUnderline = function() {
    document.execCommand('underline', false, null);
}

RE.setTextColor = function(color) {
    RE.selection.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('foreColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setTextBackgroundColor = function(color) {
    RE.selection.restorerange();
    document.execCommand("styleWithCSS", null, true);
    document.execCommand('hiliteColor', false, color);
    document.execCommand("styleWithCSS", null, false);
}

RE.setHeading = function(heading) {
    document.execCommand('formatBlock', false, '<h'+heading+'>');
}

RE.setIndent = function() {
    document.execCommand('indent', false, null);
}

RE.setOutdent = function() {
    document.execCommand('outdent', false, null);
}

RE.setJustifyLeft = function() {
    document.execCommand('justifyLeft', false, null);
}

RE.setJustifyCenter = function() {
    document.execCommand('justifyCenter', false, null);
}

RE.setJustifyRight = function() {
    document.execCommand('justifyRight', false, null);
}

RE.setOrderedList = function() {
    document.execCommand('insertOrderedList', false, null);
}

RE.setUnorderedList = function() {
    document.execCommand('insertUnorderedList', false, null);
}

RE.setBlockquote = function() {
    document.execCommand('formatBlock', false, '<blockquote>');
}

RE.insertImage = function(url, alt) {
    var html = '<p><img class="image" imageid="imageid_'+url+'" src="' + url + '" alt="' + alt + '" /></p>';
    RE.insertHTML(html);
}

RE.insertVideo = function(url) {
    var html = '<video src="'+url+'" controls autoplay><source type=video/mp4 />Your browser does not support the video tag.</video>';
    RE.insertHTML(html);
}

RE.insertAudio = function(url){
    var html = '<audio src="'+url+'" controls="controls" />';
    RE.insertHTML(html);
}

RE.insertHTML = function(html) {
    RE.selection.restorerange();
    document.execCommand('insertHTML', false, html);
}

RE.insertLink = function(url, title) {
    RE.selection.restorerange();
    var sel = document.getSelection();
    if (sel.toString().length != 0) {
        if (sel.rangeCount) {

            var el = document.createElement("a");
            el.setAttribute("href", url);
            el.setAttribute("title", title);

            var range = sel.getRangeAt(0).cloneRange();
            range.surroundContents(el);
            sel.removeAllRanges();
            sel.addRange(range);
        }
    }
    RE.callback();
}

RE.focus = function() {
    var range = document.createRange();
    range.selectNodeContents(RE.editor);
    range.collapse(false);
    var selection = window.getSelection();
    selection.removeAllRanges();
    selection.addRange(range);
    RE.editor.focus();
}

RE.blurFocus = function() {
    RE.editor.blur();
}

RE.selection.prepareInsert = function() {
    RE.selection.backuprange();
}

RE.selection.backuprange = function(){
    var selection = window.getSelection();
    var range = selection.getRangeAt(0);
    RE.currentSelection = {
        "startContainer": range.startContainer,
        "startOffset": range.startOffset,
        "endContainer": range.endContainer,
        "endOffset": range.endOffset};
}

RE.selection.restorerange = function(){
    var selection = window.getSelection();
    selection.removeAllRanges();
    var range = document.createRange();
    range.setStart(RE.currentSelection.startContainer, RE.currentSelection.startOffset);
    range.setEnd(RE.currentSelection.endContainer, RE.currentSelection.endOffset);
    selection.addRange(range);
}

/**
 * Starts the touch and saves the given x and y coordinates as last touch point
 */
RE.selection.startTouch = function(x, y) {
    RE.selection.lastTouchPoint = {'x': x, 'y': y};
};

/**
 *  Checks to see if there is a selection.
 *
 *  @return boolean
 */
RE.selection.hasSelection = function() {
    return window.getSelection().toString().length > 0;
};

/**
 *  Clears the current selection.
 */
RE.selection.clearSelection = function() {
    try {
        // if current selection clear it.
        var sel = window.getSelection();
        sel.removeAllRanges();
    }
    catch (e) {
        window.TextSelection.jsError(e);
    }
};

/**
 *  Handles the long touch action by selecting the last touched element.
 */
RE.selection.longTouch = function() {
    try {
        RE.selection.clearSelection();
        var sel = window.getSelection();
        var range = document.caretRangeFromPoint(RE.selection.lastTouchPoint.x, RE.selection.lastTouchPoint.y);
        range.expand("word");
        var text = range.toString();
        if (text.length == 1) {
            var baseKind = jpntext.kind(text);
            if (baseKind != jpntext.KIND['ascii']) {
                try {
                    do {
                        range.setEnd(range.endContainer, range.endOffset + 1);
                        text = range.toString();
                        var kind = jpntext.kind(text);
                    } while (baseKind == kind);
                    range.setEnd(range.endContainer, range.endOffset - 1);
                }
                catch (e) {
                }
                try {
                    do {
                        range.setStart(range.startContainer, range.startOffset - 1);
                        text = range.toString();
                        var kind = jpntext.kind(text);
                    } while (baseKind == kind);
                    range.setStart(range.startContainer, range.startOffset + 1);
                }
                catch (e) {
                }
            }
        }
        if (text.length > 0) {
            sel.addRange(range);
            RE.selection.saveSelectionStart();
            RE.selection.saveSelectionEnd();
            RE.selection.selectionChanged(true);
        }
     }
     catch (err) {
        window.TextSelection.jsError(err);
     }
};

/**
 * Tells the app to show the context menu.
 */
RE.selection.selectionChanged = function(isReallyChanged) {
    try {
        var sel = window.getSelection();
        if (!sel) {
            return;
        }
        var range = sel.getRangeAt(0);

        // Add spans to the selection to get page offsets
        var selectionStart = $("<span id=\"selectionStart\">&#xfeff;</span>");
        var selectionEnd = $("<span id=\"selectionEnd\"></span>");

        var startRange = document.createRange();
        startRange.setStart(range.startContainer, range.startOffset);
        startRange.insertNode(selectionStart[0]);

        var endRange = document.createRange();
        endRange.setStart(range.endContainer, range.endOffset);
        endRange.insertNode(selectionEnd[0]);

        var handleBounds = "{'left': " + (selectionStart.offset().left) + ", ";
        handleBounds += "'top': " + (selectionStart.offset().top + selectionStart.height()) + ", ";
        handleBounds += "'right': " + (selectionEnd.offset().left) + ", ";
        handleBounds += "'bottom': " + (selectionEnd.offset().top + selectionEnd.height()) + "}";

        // Pull the spans
        selectionStart.remove();
        selectionEnd.remove();

        // Reset range
        sel.removeAllRanges();
        sel.addRange(range);

        // Rangy
        var rangyRange = RE.selection.getRange();

        // Text to send to the selection
        var text = window.getSelection().toString();

        // Set the content width
        window.TextSelection.setContentWidth(document.body.clientWidth);

        // Tell the interface that the selection changed
        window.TextSelection.selectionChanged(rangyRange, text, handleBounds, isReallyChanged);
    }
    catch (e) {
        window.TextSelection.jsError(e);
    }
};

RE.selection.getRange = function() {
    var serializedRangeSelected = rangy.serializeSelection();
    var serializerModule = rangy.modules.Serializer;
    if (serializedRangeSelected != '') {
        if (rangy.supported && serializerModule && serializerModule.supported) {
            var beginingCurly = serializedRangeSelected.indexOf("{");
            serializedRangeSelected = serializedRangeSelected.substring(0, beginingCurly);
            return serializedRangeSelected;
        }
    }
}

/**
 * Returns the last touch point as a readable string.
 */
RE.selection.lastTouchPointString = function(){
    if (RE.selection.lastTouchPoint == null)
        return "undefined";
    return "{" + RE.selection.lastTouchPoint.x + "," + RE.selection.lastTouchPoint.y + "}";
};

RE.selection.saveSelectionStart = function(){
    try {
        // Save the starting point of the selection
        var sel = window.getSelection();
        var range = sel.getRangeAt(0);
        var saveRange = document.createRange();
        saveRange.setStart(range.startContainer, range.startOffset);
        RE.selection.selectionStartRange = saveRange;
    }
    catch (e) {
        window.TextSelection.jsError(e);
    }
};

RE.selection.saveSelectionEnd = function(){
    try {
        // Save the end point of the selection
        var sel = window.getSelection();
        var range = sel.getRangeAt(0);
        var saveRange = document.createRange();
        saveRange.setStart(range.endContainer, range.endOffset);
        RE.selection.selectionEndRange = saveRange;
    }
    catch (e) {
        window.TextSelection.jsError(e);
    }
};

/**
 * Sets the last caret position for the start handle.
 */
RE.selection.setStartPos = function(x, y){
    try {
        RE.selection.selectBetweenHandles(document.caretRangeFromPoint(x, y), RE.selection.selectionEndRange);
    }
    catch (e) {
        window.TextSelection.jsError(e);
    }
};

/**
 * Sets the last caret position for the end handle.
 */
RE.selection.setEndPos = function(x, y){
    try {
        RE.selection.selectBetweenHandles(RE.selection.selectionStartRange, document.caretRangeFromPoint(x, y));
    }
    catch (e) {
        window.TextSelection.jsError(e);
    }
};

RE.selection.restoreStartEndPos = function() {
    try {
        RE.selection.selectBetweenHandles(RE.selection.selectionEndRange, RE.selection.selectionStartRange);
    }
    catch (e) {
        window.TextSelection.jsError(e);
    }
};

/**
 *  Selects all content between the two handles
 */
RE.selection.selectBetweenHandles = function(startCaret, endCaret) {
    try {
        if (startCaret && endCaret) {
            var rightOrder = startCaret.compareBoundaryPoints(Range.START_TO_END, endCaret) <= 0;
            if (rightOrder) {
                RE.selection.selectionStartRange = startCaret;
                RE.selection.selectionEndRange = endCaret;
            }
            else {
                startCaret = RE.selection.selectionStartRange;
                endCaret = RE.selection.selectionEndRange;
            }
            var range = document.createRange();
            range.setStart(startCaret.startContainer, startCaret.startOffset);
            range.setEnd(endCaret.startContainer, endCaret.startOffset);
            RE.selection.clearSelection();
            var selection = window.getSelection();
            selection.addRange(range);
            RE.selection.selectionChanged(rightOrder);
        }
        else {
            RE.selection.selectionStartRange = startCaret;
            RE.selection.selectionEndRange = endCaret;
        }
    }
    catch (e) {
        window.TextSelection.jsError(e);
    }
};